#!/usr/bin/perl

use strict;
use warnings;
use Encode;
use URI::Escape;

my %kanji_freq  = ();
my $kanji_count = 0;
my %kanji_lists = ();
my %kanji_cover = ();

sub load_frequncy {
    my $freq_fname  = shift;
    my $freq_min    = shift;
    
    open(my $file, "<", $freq_fname) or die("Cannot open $freq_fname for read");    
    binmode $file, ":utf8";
    
    %kanji_freq     = ();    
    $kanji_count    = 0;      
    
    while (<$file>) {
        chomp;
        (my $kanji, my $count) = split(/,/);       
        if ($count >= $freq_min) {        
            $kanji_freq{$kanji} = $count;
        }
        $kanji_count += $count;        
    }        
    
    close($file);     
}

sub load_table {
    my $list_fname = shift;
    
    open(my $file, "<", $list_fname) or die("Cannot open $list_fname for read");
    binmode $file, ":utf8";
    
    my %list_table  = ();
    my $list_name   = <$file>;
    my $list_data   = <$file>;
    chomp($list_name);    
    chomp($list_data);
    
    foreach (split(//, $list_data)) {
        $list_table{$_} = 1;
    }    
    
    close($file);

    $kanji_lists{$list_name} = \%list_table;
}

sub generate_table {
    #
    # Report Begin
    #    
    
    print(qq^<html><head><title>Kanji Table - FooSoft.net</title><meta http-equiv="Content-Type" content="text/html; charset=utf-8"></head><body>\n^);
    print("<h1>Wikipedia Kanji Frequency Report</h1><br/>\n");
    
    
    #
    # Character Frequency and List Inclusion
    #
        
    print("<h2>Character Frequency and List Inclusion</h2>\n");
    print("<table border><tr><th>Rank</th><th>Character</th><th>Frequency</th><th>Dominance</th>\n");
    my @lists_keys = sort { $kanji_lists{$a} <=> $kanji_lists{$b} } keys(%kanji_lists);    
    foreach (@lists_keys) {
        print("<th>$_</th>");
    }    
    print("</tr>\n");
        
    my @freq_keys   = reverse sort { $kanji_freq{$a} <=> $kanji_freq{$b} } keys(%kanji_freq);
    my $row         = 1;
    
    foreach (@freq_keys) {
        my $kanji   = $_;
        my $freq    = $kanji_freq{$kanji};
        my $cover   = sprintf("%.8f", $freq / $kanji_count * 100);
        my $link    = qq^<a href = "http://jisho.org/kanji/details/^ . uri_escape_utf8($kanji) . qq^">$kanji</a>^;
        
        print("<tr><td>$row</td><td>$link</td><td>$freq</td><td>$cover%</td>");
        foreach (@lists_keys) {
            my %list = %{$kanji_lists{$_}};
            
            unless (exists($kanji_cover{$_})) {
                $kanji_cover{$_} = 0;
            }

            my $text;
            if (exists($list{$kanji})) {
                $text = qq^<font color = "#008000">yes</font>^;
                $kanji_cover{$_} += $freq;
            }
            else {
                $text = qq^<font color = "#800000">no</font>^;
            }    
                    
            print("<td>$text</td>");
        }                   
        printf("</tr>\n");
        
        ++$row;
    }    
    
    print("</table>\n");
    
    
    #
    # Intermission
    #
    
    print("<br/>\n");
    
    
    #
    # List Inclusion Summary
    #
       
    print("<h2>List Inclusion Summary</h2>\n");
    print("<table border><tr><th>List</th><th>Coverage</th></tr>\n");
    foreach (@lists_keys) {
        my $cover = sprintf("%.8f", $kanji_cover{$_} / $kanji_count * 100);
        print("<tr><td>$_</td><td>$cover%</td></tr>\n");
    }
    print("</table>\n");
    
    
    #
    # Report End
    #
    
    print(qq^<br/><center>Generated by <a href = "http://foosoft.net/contact">Alex Yatskov</a> from <a href = "http://foosoft.net">FooSoft.net</a></center></body></html>^);
};

binmode STDOUT, ":utf8";
if ($#ARGV >= 1) {
    &load_frequncy($ARGV[0], $ARGV[1]);
    if ($#ARGV >= 2) {
        my @tables = @ARGV[2 .. $#ARGV];
        foreach (@tables) {
            &load_table($_);
        }
    }
    &generate_table();
}
else {
    print("Usage: kanjitable.pl FREQ_FILE FREQ_MIN LIST_FILE...\n");
}
